@@ -16,6 +16,8 @@
 package main
 
 import (
+	"encoding/binary"
+	"encoding/hex"
 	"encoding/json"
 	"fmt"
 	"net"
@@ -468,7 +470,7 @@ func parseEvpnEthernetAutoDiscoveryArgs(args []string) (bgp.AddrPrefixInterface,
 		"rd":        paramSingle,
 		"rt":        paramList,
 		"encap":     paramSingle,
-		"esi-label": paramSingle})
+		"esi-label": paramList})
 	if err != nil {
 		return nil, nil, err
 	}
@@ -534,13 +536,14 @@ func parseEvpnMacAdvArgs(args []string) (bgp.AddrPrefixInterface, []string, erro
 		return nil, nil, fmt.Errorf("%d args required at least, but got %d", req, len(args))
 	}
 	m, err := extractReserved(args, map[string]int{
-		"esi":        paramList,
-		"etag":       paramSingle,
-		"label":      paramSingle,
-		"rd":         paramSingle,
-		"rt":         paramList,
-		"encap":      paramSingle,
-		"router-mac": paramSingle})
+		"esi":             paramList,
+		"etag":            paramSingle,
+		"label":           paramSingle,
+		"rd":              paramSingle,
+		"rt":              paramList,
+		"encap":           paramSingle,
+		"router-mac":      paramSingle,
+		"default-gateway": paramFlag})
 	if err != nil {
 		return nil, nil, err
 	}
@@ -629,11 +632,8 @@ func parseEvpnMacAdvArgs(args []string) (bgp.AddrPrefixInterface, []string, erro
 		extcomms = append(extcomms, "router-mac", m["router-mac"][0])
 	}
 
-	for _, a := range args {
-		if a == "default-gateway" {
-			extcomms = append(extcomms, "default-gateway")
-			break
-		}
+	if _, ok := m["default-gateway"]; ok {
+		extcomms = append(extcomms, "default-gateway")
 	}
 
 	r := &bgp.EVPNMacIPAdvertisementRoute{
@@ -1121,6 +1121,35 @@ func parseMUPDirectSegmentDiscoveryRouteArgs(args []string, afi uint16, nexthop
 	return bgp.NewMUPNLRI(afi, bgp.MUP_ARCH_TYPE_UNDEFINED, bgp.MUP_ROUTE_TYPE_DIRECT_SEGMENT_DISCOVERY, r), psid, extcomms, nil
 }
 
+func parseTeid(s string) (teid netip.Addr, err error) {
+	// Hex format
+	if s, ok := strings.CutPrefix(s, "0x"); ok {
+		b, err := hex.DecodeString(s)
+		if err != nil {
+			return teid, err
+		}
+		if len(b) < 4 {
+			b = append(b, make([]byte, 4-len(b))...)
+		}
+		if teid, ok = netip.AddrFromSlice(b); ok {
+			return teid, nil
+		}
+	}
+	// IP address format
+	if teid, err = netip.ParseAddr(s); err == nil {
+		return teid, err
+	}
+	// Decimal format
+	b := [4]byte{}
+	n, err := strconv.ParseUint(s, 10, 32)
+	if err != nil {
+		return teid, err
+	}
+	binary.BigEndian.PutUint32(b[:], uint32(n))
+	teid = netip.AddrFrom4(b)
+	return teid, nil
+}
+
 func parseMUPType1SessionTransformedRouteArgs(args []string, afi uint16) (bgp.AddrPrefixInterface, *bgp.PathAttributePrefixSID, []string, error) {
 	// Format:
 	// <ip prefix> rd <rd> [rt <rt>...] teid <teid> qfi <qfi> endpoint <endpoint>
@@ -1154,13 +1183,10 @@ func parseMUPType1SessionTransformedRouteArgs(args []string, afi uint16) (bgp.Ad
 	if err != nil {
 		return nil, nil, nil, err
 	}
-	teid, err := strconv.ParseUint(m["teid"][0], 10, 32)
+	teid, err := parseTeid(m["teid"][0])
 	if err != nil {
 		return nil, nil, nil, err
 	}
-	if teid == 0 {
-		return nil, nil, nil, fmt.Errorf("teid should not be 0")
-	}
 	qfi, err := strconv.ParseUint(m["qfi"][0], 10, 8)
 	if err != nil {
 		return nil, nil, nil, err
@@ -1178,7 +1204,7 @@ func parseMUPType1SessionTransformedRouteArgs(args []string, afi uint16) (bgp.Ad
 	r := &bgp.MUPType1SessionTransformedRoute{
 		RD:                    rd,
 		Prefix:                prefix,
-		TEID:                  uint32(teid),
+		TEID:                  teid,
 		QFI:                   uint8(qfi),
 		EndpointAddressLength: uint8(ea.BitLen()),
 		EndpointAddress:       ea,
@@ -1188,16 +1214,17 @@ func parseMUPType1SessionTransformedRouteArgs(args []string, afi uint16) (bgp.Ad
 
 func parseMUPType2SessionTransformedRouteArgs(args []string, afi uint16) (bgp.AddrPrefixInterface, *bgp.PathAttributePrefixSID, []string, error) {
 	// Format:
-	// <endpoint address> rd <rd> [rt <rt>...] teid <teid> [mup <segment identifier>]
-	req := 5
+	// <endpoint address> rd <rd> [rt <rt>...] endpoint-address-length <endpoint-address-length> teid <teid> [mup <segment identifier>]
+	req := 6
 	if len(args) < req {
 		return nil, nil, nil, fmt.Errorf("%d args required at least, but got %d", req, len(args))
 	}
 	m, err := extractReserved(args, map[string]int{
-		"rd":   paramSingle,
-		"rt":   paramSingle,
-		"teid": paramSingle,
-		"mup":  paramSingle,
+		"rd":                      paramSingle,
+		"rt":                      paramSingle,
+		"endpoint-address-length": paramSingle,
+		"teid":                    paramSingle,
+		"mup":                     paramSingle,
 	})
 	if err != nil {
 		return nil, nil, nil, err
@@ -1205,7 +1232,7 @@ func parseMUPType2SessionTransformedRouteArgs(args []string, afi uint16) (bgp.Ad
 	if len(m[""]) < 1 {
 		return nil, nil, nil, fmt.Errorf("specify endpoint")
 	}
-	for _, f := range []string{"rd", "rt", "teid", "mup"} {
+	for _, f := range []string{"rd", "rt", "endpoint-address-length", "teid", "mup"} {
 		for len(m[f]) == 0 {
 			return nil, nil, nil, fmt.Errorf("specify %s", f)
 		}
@@ -1218,12 +1245,16 @@ func parseMUPType2SessionTransformedRouteArgs(args []string, afi uint16) (bgp.Ad
 	if err != nil {
 		return nil, nil, nil, err
 	}
-	teid, err := strconv.ParseUint(m["teid"][0], 10, 32)
+	eaLen, err := strconv.ParseUint(m["endpoint-address-length"][0], 10, 8)
 	if err != nil {
 		return nil, nil, nil, err
 	}
-	if teid == 0 {
-		return nil, nil, nil, fmt.Errorf("teid should not be 0")
+	if (ea.Is4() && eaLen > 64) || (ea.Is6() && eaLen > 160) {
+		return nil, nil, nil, fmt.Errorf("endpoint-address-length too large: %d", eaLen)
+	}
+	teid, err := parseTeid(m["teid"][0])
+	if err != nil {
+		return nil, nil, nil, err
 	}
 	extcomms := make([]string, 0)
 	if len(m["rt"]) > 0 {
@@ -1236,9 +1267,9 @@ func parseMUPType2SessionTransformedRouteArgs(args []string, afi uint16) (bgp.Ad
 
 	r := &bgp.MUPType2SessionTransformedRoute{
 		RD:                    rd,
-		EndpointAddressLength: uint8(ea.BitLen()),
+		EndpointAddressLength: uint8(eaLen),
 		EndpointAddress:       ea,
-		TEID:                  uint32(teid),
+		TEID:                  teid,
 	}
 	return bgp.NewMUPNLRI(afi, bgp.MUP_ARCH_TYPE_UNDEFINED, bgp.MUP_ROUTE_TYPE_TYPE_2_SESSION_TRANSFORMED, r), nil, extcomms, nil
 }
@@ -1262,6 +1293,265 @@ func parseMUPArgs(args []string, afi uint16, nexthop string) (bgp.AddrPrefixInte
 	return nil, nil, nil, fmt.Errorf("invalid subtype. expect [isd|dsd|t1st|t2st] but %s", subtype)
 }
 
+func parseLsLinkProtocol(args []string, afi uint16) (bgp.AddrPrefixInterface, *bgp.PathAttributeLs, error) {
+	if len(args) < 2 {
+		return nil, nil, fmt.Errorf("lack of protocolType")
+	}
+	protocolType := args[1]
+	switch protocolType {
+	// TODO case ospf/isis
+	case "bgp":
+		return parseLsLinkNLRIType(args, afi)
+	}
+	return nil, nil, fmt.Errorf("invalid protocolType. expect [bgp] but %s", protocolType)
+}
+
+func parseLsLinkNLRIType(args []string, afi uint16) (bgp.AddrPrefixInterface, *bgp.PathAttributeLs, error) {
+	// Format:
+	// <ip prefix> identifier <identifier> asn <asn> bgp-ls-id <bgp-ls-id> ospf
+	req := 26
+	if len(args) < req {
+		return nil, nil, fmt.Errorf("%d args required at least, but got %d", req, len(args))
+	}
+
+	m, err := extractReserved(args, map[string]int{
+		"identifier":                      paramSingle,
+		"local-asn":                       paramSingle,
+		"local-bgp-ls-id":                 paramSingle,
+		"local-bgp-router-id":             paramSingle,
+		"local-bgp-confederation-member":  paramSingle,
+		"remote-asn":                      paramSingle,
+		"remote-bgp-ls-id":                paramSingle,
+		"remote-bgp-router-id":            paramSingle,
+		"remote-bgp-confederation-member": paramSingle,
+		"ipv4-interface-address":          paramSingle,
+		"ipv4-neighbor-address":           paramSingle,
+		"ipv6-interface-address":          paramSingle,
+		"ipv6-neighbor-address":           paramSingle,
+		"sid":                             paramSingle,
+		"sid-type":                        paramSingle,
+		"v-flag":                          paramFlag,
+		"l-flag":                          paramFlag,
+		"b-flag":                          paramFlag,
+		"p-flag":                          paramFlag,
+		"weight":                          paramSingle,
+	})
+	if err != nil {
+		return nil, nil, err
+	}
+
+	identifier, err := strconv.ParseUint(m["identifier"][0], 10, 64)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	localAsn, err := strconv.ParseUint(m["local-asn"][0], 10, 64)
+	if err != nil {
+		return nil, nil, err
+
+	}
+	localBgpLsId, err := strconv.ParseUint(m["local-bgp-ls-id"][0], 10, 64)
+	if err != nil {
+		return nil, nil, err
+
+	}
+	localBgpConfederationMember, err := strconv.ParseUint(m["local-bgp-confederation-member"][0], 10, 64)
+	if err != nil {
+		return nil, nil, err
+
+	}
+	lnd := &bgp.LsNodeDescriptor{
+		Asn:                    uint32(localAsn),
+		BGPLsID:                uint32(localBgpLsId),
+		OspfAreaID:             0,
+		PseudoNode:             false,
+		IGPRouterID:            "",
+		BGPRouterID:            net.ParseIP(m["local-bgp-router-id"][0]),
+		BGPConfederationMember: uint32(localBgpConfederationMember),
+	}
+	RemoteAsn, err := strconv.ParseUint(m["remote-asn"][0], 10, 64)
+	if err != nil {
+		return nil, nil, err
+
+	}
+	RemoteBgpLsId, err := strconv.ParseUint(m["remote-bgp-ls-id"][0], 10, 64)
+	if err != nil {
+		return nil, nil, err
+
+	}
+	RemoteBgpConfederationMember, err := strconv.ParseUint(m["remote-bgp-confederation-member"][0], 10, 64)
+	if err != nil {
+		return nil, nil, err
+
+	}
+	rnd := &bgp.LsNodeDescriptor{
+		Asn:                    uint32(RemoteAsn),
+		BGPLsID:                uint32(RemoteBgpLsId),
+		OspfAreaID:             0,
+		PseudoNode:             false,
+		IGPRouterID:            "",
+		BGPRouterID:            net.ParseIP(m["remote-bgp-router-id"][0]),
+		BGPConfederationMember: uint32(RemoteBgpConfederationMember),
+	}
+
+	var interfaceAddrIPv4 net.IP
+	var neighborAddrIPv4 net.IP
+	var interfaceAddrIPv6 net.IP
+	var neighborAddrIPv6 net.IP
+
+	interfaceAddrIPv4 = net.ParseIP(m["ipv4-interface-address"][0]).To4()
+	neighborAddrIPv4 = net.ParseIP(m["ipv4-neighbor-address"][0]).To4()
+	interfaceAddrIPv6 = net.ParseIP(m["ipv6-interface-address"][0]).To16()
+	neighborAddrIPv6 = net.ParseIP(m["ipv6-neighbor-address"][0]).To16()
+
+	ld := &bgp.LsLinkDescriptor{
+		LinkLocalID:       new(uint32),
+		LinkRemoteID:      new(uint32),
+		InterfaceAddrIPv4: &interfaceAddrIPv4,
+		NeighborAddrIPv4:  &neighborAddrIPv4,
+		InterfaceAddrIPv6: &interfaceAddrIPv6,
+		NeighborAddrIPv6:  &neighborAddrIPv6,
+	}
+
+	lndTLV := bgp.NewLsTLVNodeDescriptor(lnd, bgp.LS_TLV_LOCAL_NODE_DESC)
+	rndTLV := bgp.NewLsTLVNodeDescriptor(rnd, bgp.LS_TLV_REMOTE_NODE_DESC)
+	ldTLV := bgp.NewLsLinkTLVs(ld)
+
+	sidTypeString := m["sid-type"][0]
+
+	sidtype := lsTLVTypeSelect(sidTypeString)
+
+	var peerNodeFlag uint8
+
+	if _, ok := m["v-flag"]; ok {
+		peerNodeFlag = peerNodeFlag | 0x80
+	}
+	if _, ok := m["l-flag"]; ok {
+		peerNodeFlag = peerNodeFlag | 0x40
+	}
+	if _, ok := m["b-flag"]; ok {
+		peerNodeFlag = peerNodeFlag | 0x20
+	}
+	if _, ok := m["p-flag"]; ok {
+		peerNodeFlag = peerNodeFlag | 0x10
+	}
+
+	lsTLVWeight, err := strconv.ParseUint(m["weight"][0], 10, 64)
+	if err != nil {
+		return nil, nil, err
+	}
+	lsTLVSid, err := strconv.ParseUint(m["sid"][0], 10, 64)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	const lsTlvLen = 7
+	const t = bgp.BGP_ATTR_TYPE_LS
+	const pathAttrHdrLen = 4
+	var tlvs []bgp.LsTLVInterface
+	length := uint16(pathAttrHdrLen + lsTlvLen)
+
+	switch sidtype {
+	case bgp.LS_TLV_PEER_NODE_SID:
+
+		lsTLV := &bgp.LsTLVPeerNodeSID{
+			LsTLV: bgp.LsTLV{
+				Type:   sidtype,
+				Length: uint16(lsTlvLen),
+			},
+			Flags:  peerNodeFlag,
+			Weight: uint8(lsTLVWeight),
+			SID:    uint32(lsTLVSid),
+		}
+		tlvs = append(tlvs, lsTLV)
+	case bgp.LS_TLV_ADJACENCY_SID:
+		lsTLV := &bgp.LsTLVAdjacencySID{
+			LsTLV: bgp.LsTLV{
+				Type:   sidtype,
+				Length: uint16(lsTlvLen),
+			},
+			Flags:  peerNodeFlag,
+			Weight: uint8(lsTLVWeight),
+			SID:    uint32(lsTLVSid),
+		}
+		tlvs = append(tlvs, lsTLV)
+	case bgp.LS_TLV_PEER_SET_SID:
+		lsTLV := &bgp.LsTLVPeerSetSID{
+			LsTLV: bgp.LsTLV{
+				Type:   sidtype,
+				Length: uint16(lsTlvLen),
+			},
+			Flags:  peerNodeFlag,
+			Weight: uint8(lsTLVWeight),
+			SID:    uint32(lsTLVSid),
+		}
+		tlvs = append(tlvs, lsTLV)
+	}
+
+	pathAttributeLs := &bgp.PathAttributeLs{
+		PathAttribute: bgp.PathAttribute{
+			Flags:  bgp.PathAttrFlags[t],
+			Type:   t,
+			Length: length,
+		},
+		TLVs: tlvs,
+	}
+	len := len(ldTLV)
+	var sum int
+	for i := 0; i < len; i++ {
+		sum += ldTLV[i].Len()
+	}
+
+	const CodeLen = 1
+	const topologyLen = 8
+	LsNLRIhdrlen := sum + lndTLV.Len() + rndTLV.Len() + topologyLen + CodeLen
+	lsNlri := bgp.LsNLRI{
+		NLRIType:   bgp.LS_NLRI_TYPE_NODE,
+		Length:     uint16(LsNLRIhdrlen),
+		ProtocolID: 7,
+		Identifier: identifier,
+	}
+	nlri := &bgp.LsAddrPrefix{
+		Type:   bgp.LS_NLRI_TYPE_NODE,
+		Length: 4,
+		NLRI: &bgp.LsLinkNLRI{
+			LsNLRI:         lsNlri,
+			LocalNodeDesc:  &lndTLV,
+			RemoteNodeDesc: &rndTLV,
+			LinkDesc:       ldTLV,
+		},
+	}
+	return nlri, pathAttributeLs, nil
+}
+
+func lsTLVTypeSelect(s string) bgp.LsTLVType {
+	switch s {
+	case "node":
+		return bgp.LS_TLV_PEER_NODE_SID
+	case "adj":
+		return bgp.LS_TLV_ADJACENCY_SID
+	case "set":
+		return bgp.LS_TLV_PEER_SET_SID
+	}
+
+	return bgp.LS_TLV_UNKNOWN
+
+}
+
+func parseLsArgs(args []string, afi uint16) (bgp.AddrPrefixInterface, *bgp.PathAttributeLs, error) {
+	if len(args) < 1 {
+		return nil, nil, fmt.Errorf("lack of nlriType")
+	}
+	nlriType := args[0]
+	switch nlriType {
+	case "link":
+		return parseLsLinkNLRIType(args, afi)
+		// TODO: case node / IPv4 Topology Prefix / IPv6 Topology Prefix / TE Policy / SRv6 SID
+	}
+
+	return nil, nil, fmt.Errorf("invalid nlriType. expect [link] but %s", nlriType)
+}
+
 func extractOrigin(args []string) ([]string, bgp.PathAttributeInterface, error) {
 	typ := bgp.BGP_ORIGIN_ATTR_TYPE_INCOMPLETE
 	for idx, arg := range args {
@@ -1503,6 +1793,7 @@ func parsePath(rf bgp.RouteFamily, args []string) (*api.Path, error) {
 	var nlri bgp.AddrPrefixInterface
 	var extcomms []string
 	var psid *bgp.PathAttributePrefixSID
+	var ls *bgp.PathAttributeLs
 	var err error
 	attrs := make([]bgp.PathAttributeInterface, 0, 1)
 
@@ -1653,12 +1944,17 @@ func parsePath(rf bgp.RouteFamily, args []string) (*api.Path, error) {
 		nlri, psid, extcomms, err = parseMUPArgs(args, bgp.AFI_IP, nexthop)
 	case bgp.RF_MUP_IPv6:
 		nlri, psid, extcomms, err = parseMUPArgs(args, bgp.AFI_IP6, nexthop)
+	case bgp.RF_LS:
+		nlri, ls, err = parseLsArgs(args, bgp.AFI_LS)
 	default:
 		return nil, fmt.Errorf("unsupported route family: %s", rf)
 	}
 	if err != nil {
 		return nil, err
 	}
+	if ls != nil {
+		attrs = append(attrs, ls)
+	}
 
 	if rf == bgp.RF_IPv4_UC && net.ParseIP(nexthop).To4() != nil {
 		attrs = append(attrs, bgp.NewPathAttributeNextHop(nexthop))
@@ -1740,10 +2036,10 @@ func modPath(resource string, name, modtype string, args []string) error {
 		helpErrMap := map[bgp.RouteFamily]error{}
 		baseHelpMsgFmt := fmt.Sprintf(`error: %s
 usage: %s rib -a %%s %s <PREFIX> %%s [origin { igp | egp | incomplete }] [aspath <ASPATH>] [nexthop <ADDRESS>] [med <NUM>] [local-pref <NUM>] [community <COMMUNITY>] [aigp metric <NUM>] [large-community <LARGE_COMMUNITY>] [aggregator <AGGREGATOR>]
-	<ASPATH>: <AS>[,<AS>],
-	<COMMUNITY>: xxx:xxx|internet|planned-shut|accept-own|route-filter-translated-v4|route-filter-v4|route-filter-translated-v6|route-filter-v6|llgr-stale|no-llgr|blackhole|no-export|no-advertise|no-export-subconfed|no-peer,
-	<LARGE_COMMUNITY>: xxx:xxx:xxx[,<LARGE_COMMUNITY>],
-	<AGGREGATOR>: <AS>:<ADDRESS>`,
+    <ASPATH>: <AS>[,<AS>],
+    <COMMUNITY>: xxx:xxx|internet|planned-shut|accept-own|route-filter-translated-v4|route-filter-v4|route-filter-translated-v6|route-filter-v6|llgr-stale|no-llgr|blackhole|no-export|no-advertise|no-export-subconfed|no-peer,
+    <LARGE_COMMUNITY>: xxx:xxx:xxx[,<LARGE_COMMUNITY>],
+    <AGGREGATOR>: <AS>:<ADDRESS>`,
 			err,
 			cmdstr,
 			// <address family>
@@ -1871,7 +2167,7 @@ usage: %s rib %s { isd <ISD> | dsd <DSD> | t1st <T1ST> | t2st <T2ST> } -a mup-ip
     <ISD>  : <ip prefix> rd <rd> prefix <prefix> locator-node-length <locator-node-length> function-length <function-length> behavior <behavior> [rt <rt>...]
     <DSD>  : <ip address> rd <rd> prefix <prefix> locator-node-length <locator-node-length> function-length <function-length> behavior <behavior> [rt <rt>...] [mup <segment identifier>]
     <T1ST> : <ip prefix> rd <rd> [rt <rt>...] teid <teid> qfi <qfi> endpoint <endpoint>
-    <T2ST> : <endpoint address> rd <rd> [rt <rt>...] teid <teid> [mup <segment identifier>]`,
+    <T2ST> : <endpoint address> rd <rd> [rt <rt>...] endpoint-address-length <endpoint-address-length> teid <teid> [mup <segment identifier>]`,
 			err,
 			cmdstr,
 			modtype,
@@ -1881,7 +2177,7 @@ usage: %s rib %s { isd <ISD> | dsd <DSD> | t1st <T1ST> | t2st <T2ST> } -a mup-ip
     <ISD>  : <ip prefix> rd <rd> prefix <prefix> locator-node-length <locator-node-length> function-length <function-length> behavior <behavior> [rt <rt>...]
     <DSD>  : <ip address> rd <rd> prefix <prefix> locator-node-length <locator-node-length> function-length <function-length> behavior <behavior> [rt <rt>...] [mup <segment identifier>]
     <T1ST> : <ip prefix> rd <rd> [rt <rt>...] teid <teid> qfi <qfi> endpoint <endpoint>
-    <T2ST> : <endpoint address> rd <rd> [rt <rt>...] teid <teid> [mup <segment identifier>]`,
+    <T2ST> : <endpoint address> rd <rd> [rt <rt>...] endpoint-address-length <endpoint-address-length> teid <teid> [mup <segment identifier>]`,
 			err,
 			cmdstr,
 			modtype,
