@@ -466,20 +466,21 @@ func NewPeerFromConfigStruct(pconf *Neighbor) *api.Peer {
 	return &api.Peer{
 		ApplyPolicy: newApplyPolicyFromConfigStruct(&pconf.ApplyPolicy),
 		Conf: &api.PeerConf{
-			NeighborAddress:   pconf.Config.NeighborAddress,
-			PeerAsn:           pconf.Config.PeerAs,
-			LocalAsn:          pconf.Config.LocalAs,
-			Type:              api.PeerType(pconf.Config.PeerType.ToInt()),
-			AuthPassword:      pconf.Config.AuthPassword,
-			RouteFlapDamping:  pconf.Config.RouteFlapDamping,
-			Description:       pconf.Config.Description,
-			PeerGroup:         pconf.Config.PeerGroup,
-			NeighborInterface: pconf.Config.NeighborInterface,
-			Vrf:               pconf.Config.Vrf,
-			AllowOwnAsn:       uint32(pconf.AsPathOptions.Config.AllowOwnAs),
-			RemovePrivate:     removePrivate,
-			ReplacePeerAsn:    pconf.AsPathOptions.Config.ReplacePeerAs,
-			AdminDown:         pconf.Config.AdminDown,
+			NeighborAddress:     pconf.Config.NeighborAddress,
+			PeerAsn:             pconf.Config.PeerAs,
+			LocalAsn:            pconf.Config.LocalAs,
+			Type:                api.PeerType(pconf.Config.PeerType.ToInt()),
+			AuthPassword:        pconf.Config.AuthPassword,
+			RouteFlapDamping:    pconf.Config.RouteFlapDamping,
+			Description:         pconf.Config.Description,
+			PeerGroup:           pconf.Config.PeerGroup,
+			NeighborInterface:   pconf.Config.NeighborInterface,
+			Vrf:                 pconf.Config.Vrf,
+			AllowOwnAsn:         uint32(pconf.AsPathOptions.Config.AllowOwnAs),
+			RemovePrivate:       removePrivate,
+			ReplacePeerAsn:      pconf.AsPathOptions.Config.ReplacePeerAs,
+			AdminDown:           pconf.Config.AdminDown,
+			SendSoftwareVersion: pconf.Config.SendSoftwareVersion,
 		},
 		State: &api.PeerState{
 			SessionState: api.PeerState_SessionState(api.PeerState_SessionState_value[strings.ToUpper(string(s.SessionState))]),
@@ -507,6 +508,7 @@ func NewPeerFromConfigStruct(pconf *Neighbor) *api.Peer {
 				},
 			},
 			PeerAsn:         s.PeerAs,
+			LocalAsn:        s.LocalAs,
 			Type:            api.PeerType(s.PeerType.ToInt()),
 			NeighborAddress: pconf.State.NeighborAddress,
 			Queues:          &api.Queues{},
@@ -552,6 +554,8 @@ func NewPeerFromConfigStruct(pconf *Neighbor) *api.Peer {
 			NotificationEnabled: pconf.GracefulRestart.Config.NotificationEnabled,
 			LonglivedEnabled:    pconf.GracefulRestart.Config.LongLivedEnabled,
 			LocalRestarting:     pconf.GracefulRestart.State.LocalRestarting,
+			PeerRestartTime:     uint32(pconf.GracefulRestart.State.PeerRestartTime),
+			PeerRestarting:      pconf.GracefulRestart.State.PeerRestarting,
 		},
 		Transport: &api.Transport{
 			RemotePort:    uint32(pconf.Transport.Config.RemotePort),
